@@ -3,7 +3,7 @@
 # Name     : ModProxyPerlHtml.pm
 # Language : perl 5
 # Authors  : Gilles Darold, gilles at darold dot net
-# Copyright: Copyright (c) 2005-2010: Gilles Darold - All rights reserved -
+# Copyright: Copyright (c) 2005-2012: Gilles Darold - All rights reserved -
 # Description : This mod_perl2 module is a replacement for mod_proxy_html.c
 #		with far better URL HTML rewriting.
 # Usage    : See documentation in this file with perldoc.
@@ -28,7 +28,9 @@ use constant BUFF_LEN => 8000;
 use Apache2::ServerRec;
 use Apache2::URI;
 
-$Apache2::ModProxyPerlHtml::VERSION = '3.0';
+
+$Apache2::ModProxyPerlHtml::VERSION = '3.4';
+
 
 %Apache2::ModProxyPerlHtml::linkElements = (
 	'a'       => ['href'],
@@ -44,7 +46,7 @@ $Apache2::ModProxyPerlHtml::VERSION = '3.0';
 	'iframe'  => ['src', 'longdesc'],
 	'ilayer'  => ['background'],
 	'img'     => ['src', 'lowsrc', 'longdesc', 'usemap'],
-	'input'   => ['src', 'usemap'],
+	'input'   => ['src', 'usemap','formaction'],
 	'ins'     => ['cite'],
 	'isindex' => ['action'],
 	'head'    => ['profile'],
@@ -58,6 +60,7 @@ $Apache2::ModProxyPerlHtml::VERSION = '3.0';
 	'th'      => ['background'],
 	'tr'      => ['background'],
 	'xmp'     => ['href'],
+	'button'  => ['formaction'],
 );
 
 sub handler
@@ -77,6 +80,11 @@ sub handler
 		$f->r->headers_out->unset('Content-Length');
 		my @pattern = $f->r->dir_config->get('ProxyHTMLURLMap');
 		my @rewrite = $f->r->dir_config->get('ProxyHTMLRewrite');
+		my $contenttype = $f->r->dir_config->get('ProxyHTMLContentType');
+		$contenttype ||= '(text\/javascript|text\/html|text\/css|text\/xml|application\/.*javascript|application\/.*xml)';
+		my $badcontenttype = $f->r->dir_config->get('ProxyHTMLExcludeContentType');
+		$badcontenttype ||= '(application\/vnd\.openxml)';
+
 		my $ct = $f->ctx;
 		$ct->{data} = '';
 		foreach my $p (@pattern) {
@@ -85,6 +93,8 @@ sub handler
 		foreach my $p (@rewrite) {
 			push(@{$ct->{rewrite}}, $p);
 		}
+		$ct->{contenttype} = $contenttype;
+		$ct->{badcontenttype} = $badcontenttype;
 		$f->ctx($ct);
 	}
 	# Thing we do on all invocations
@@ -98,51 +108,71 @@ sub handler
 	if ($f->seen_eos) { 
 		# Skip content that should not have links
 		my $parsed_uri = $f->r->construct_url();
-		my $encoding = $f->r->headers_in->{'Accept-Encoding'} || '';
+		my $a_encoding = $f->r->headers_in->{'Accept-Encoding'} || '';
+		my $c_encoding = $f->r->headers_out->{'Content-Encoding'} || '';
+		my $ct = $f->r->headers_out->{'Content-type'} || '';
 	       	# if Accept-Encoding: gzip,deflate try to uncompress
-		if ($encoding =~ /gzip|deflate/) {
+		if ( ($c_encoding =~ /gzip|deflate/) && ($ct =~ /$ctx->{contenttype}/is) && ($ct !~ /$ctx->{badcontenttype}/is) ) {
+			if ($debug) {
+				Apache2::ServerRec::warn("[ModProxyPerlHtml] Uncompressing $ct, Content-Encoding: $c_encoding\n");
+			}
 			use IO::Uncompress::AnyInflate qw(anyinflate $AnyInflateError) ;
 			my $output = '';
 			anyinflate  \$ctx->{data} => \$output or print STDERR "anyinflate failed: $AnyInflateError\n";
 			if ($ctx->{data} ne $output) {
 				$ctx->{data} = $output;
 			} else {
-				$encoding = '';
+				$c_encoding = '';
 			}
+		} else {
+			$c_encoding = '';
 		}
 		my $refresh = $f->r->headers_out->{'Refresh'};
 		if ($refresh) {
 			foreach my $p (@{$ctx->{pattern}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p);
-				$refresh =~ s#([^\/:])$match#$1$substitute#;
+				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
+				if ($refresh =~ s#([^\/:])$match#$1$substitute#) {
+					if ($debug) {
+						Apache2::ServerRec::warn("[ModProxyPerlHtml] Refresh header match '$match', substituted by: /$substitute/\n");
+					}
+				}
 			}
 			$f->r->headers_out->set('Refresh' => $refresh);
 		}
 		
-		if ($content_type =~ /(text\/javascript|text\/html|text\/css|application\/.*javascript)/) {
+		if ( ($content_type =~ /$ctx->{contenttype}/is) && ($content_type !~ /$ctx->{badcontenttype}/is) ) {
+			if ($debug) {
+				Apache2::ServerRec::warn("[ModProxyPerlHtml] Content-type '$content_type' match: /$ctx->{contenttype}/is\n");
+			}
 			# Replace links if pattern match
 			foreach my $p (@{$ctx->{pattern}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p);
+				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
 				&link_replacement(\$ctx->{data}, $match, $substitute, $parsed_uri);
 
 			}
 			# Rewrite code if rewrite pattern match
 			foreach my $p (@{$ctx->{rewrite}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p);
+				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
 				&rewrite_content(\$ctx->{data}, $match, $substitute, $parsed_uri);
 			}
 		}
 
-		if ($encoding =~ /gzip/) {
-			use IO::Compress::Gzip qw(gzip $GzipError) ;
-			my $output = '';
-			my $status = gzip \$ctx->{data} => \$output or die "gzip failed: $GzipError\n";
-			$ctx->{data} = $output;
-		} elsif ($encoding =~ /deflate/) {
-			use IO::Compress::Deflate qw(deflate $DeflateError) ;
-			my $output = '';
-			my $status = deflate \$ctx->{data} => \$output or die "deflate failed: $DeflateError\n";
-			$ctx->{data} = $output;
+		# Compress again data if require
+		if (($a_encoding =~ /gzip|deflate/) && ($c_encoding =~ /gzip|deflate/)) {
+			if ($debug) {
+				Apache2::ServerRec::warn("[ModProxyPerlHtml] Compressing output as Content-Encoding: $c_encoding\n");
+			}
+			if ($c_encoding =~ /gzip/) {
+				use IO::Compress::Gzip qw(gzip $GzipError) ;
+				my $output = '';
+				my $status = gzip \$ctx->{data} => \$output or die "gzip failed: $GzipError\n";
+				$ctx->{data} = $output;
+			} elsif ($c_encoding =~ /deflate/) {
+				use IO::Compress::Deflate qw(deflate $DeflateError) ;
+				my $output = '';
+				my $status = deflate \$ctx->{data} => \$output or die "deflate failed: $DeflateError\n";
+				$ctx->{data} = $output;
+			}
 		}
 		$f->ctx($ctx);
 
@@ -151,10 +181,11 @@ sub handler
 		my $c = $f->c;
 		if ($c->keepalive == Apache2::Const::CONN_KEEPALIVE && $ctx->{data} && $c->keepalives > $ctx->{keepalives}) {
 			if ($debug) {
-				warn "[ModProxyPerlHtml] cleaning context for keep alive request\n";
+				Apache2::ServerRec::warn("[ModProxyPerlHtml] Cleaning context for keep alive request\n");
 			}
 			$ctx->{data} = '';
 			$ctx->{pattern} = ();
+			$ctx->{rewrite} = ();
 			$ctx->{keepalives} = $c->keepalives;
 		}
 			
@@ -249,6 +280,17 @@ by other, like changing images name or anything else. mod_proxy_html can't do
 all of that. Since release 3.0 ModProxyPerlHtml is also able to rewrite HTTP
 headers with refresh url redirection. 
 
+The replacement capability concern only the following HTTP content type:
+
+	text/javascript
+	text/html
+	text/css
+	text/xml
+	application/.*javascript
+	application/.*xml
+
+other kind of file, will be left untouched (or see ProxyHTMLContentType and ProxyHTMLExcludeContentType).
+
 =head1 AVAIBILITY
 
 You can get the latest version of Apache2::ModProxyPerlHtml from CPAN
@@ -280,7 +322,6 @@ In your Apache configuration file you have to load the following DSO modules:
     LoadModule ssl_module modules/mod_ssl.so
     LoadModule perl_module  modules/mod_perl.so
 
-
 Then in your Apache site or virtualhost configuration file use ModProxyPerlHtml*
 as follow:
 
@@ -311,10 +352,12 @@ follow:
     <Location /webcal/>
         ...
         PerlAddVar ProxyHTMLRewrite "/logo/image1.png /images/logo1.png"
+	# Or more complicated to handle space in the code as space is the
+	# pattern / substitution separator character internally in ModProxyPerlHtml
+	PerlAddVar ProxyHTMLRewrite "ajaxurl[\s\t]*=[\s\t]*'/blog' ajaxurl = '/www2.mydom.org/blog'"
         ...
     </Location>
 
-
 this will replace each occurence of '/logo/image1.png' by '/images/logo1.png' in
 the entire stream (html, javascript or css). Note that this kind of substitution
 is done after all other proxy related replacements.
@@ -331,6 +374,32 @@ To avoid the second replacement, write your JS code like that:
 
         imgUp.src = '/images/' + varPath + unescape('%2F') + 'up.png';
 
+ModProxyPerlHTML replacement is activated on certain HTTP Content Type. If you
+experienced that replacement is not activated for your file type, you can use the
+ProxyHTMLContentType configuration directive to redefined the HTTP Content Type
+that should be parsed by ModProxyPerlHTML. The default value is the following
+Perl regular expresssion:
+
+	ProxyHTMLContentType    (text\/javascript|text\/html|text\/css|text\/xml|application\/.*javascript|application\/.*xml)
+
+If you know exactly what you are doing by editing this regexp fill free to add
+the missing Content-Type that must be parsed by ModProxyPerlHTML. Otherwise drop
+me a line with the content type, I will give you the rigth expression. If you don't
+know about the content type, with FireFox simply type Ctrl+i on the web page.
+
+Some MS Office files may conflict with the above ProxyHTMLContentType regex like .docx or .xlsx
+files. The result is that there could suffer of replacement inside and the file will be corrupted.
+to prevent this you have the ProxyHTMLExcludeContentType configuration directive to exclude certain
+content-type. Here is the default value:
+ 
+	ProxyHTMLExcludeContentType	(application\/vnd\.openxml)
+
+If you have problem with other content-type, use this directive. For example, as follow:
+
+	ProxyHTMLExcludeContentType	(application\/vnd\.openxml|application\/vnd\..*text)
+
+this regex will prevent any MS Office XML or text document to be parsed.
+
 =head1 LIVE EXAMPLE
 
 Here is the reverse proxy configuration I use to give access to Internet users
@@ -385,7 +454,7 @@ to internal applications:
     <Location /intranet/>
         ProxyPassReverse /
         PerlAddVar ProxyHTMLURLMap "/ /intranet/"
-        PerlAddVar ProxyHTMLURLMap "http://intranet.samse.fr /intranet"
+        PerlAddVar ProxyHTMLURLMap "http://intranet.domain.com /intranet"
 	# Rewrite links that give access to the two previous location 
         PerlAddVar ProxyHTMLURLMap "/intranet/webmail /webmail"
         PerlAddVar ProxyHTMLURLMap "/intranet/webcal /webcal"
@@ -404,7 +473,7 @@ requests.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2010 - Gilles Darold
+Copyright (c) 2005-2012 - Gilles Darold
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
